@@ -91,8 +91,26 @@ const std::string DeviceManager::getDevicesXML()
 }
 
 void DeviceManager::startFindDevices() {
-    // Think about putting this routine into a thread when devices will be supported that take more time to search for
+	this->findDeviceState=1;
+	int code = pthread_create(&(this->threadId), NULL, DeviceManager::findDeviceThread, (void*)this);
 
+    if (code != 0) {
+        Log::err("Creation of findDevices thread failed!");
+        this->findDeviceState=0;
+        return;  // Not really able to continue :-(
+    }
+}
+
+/*static*/
+void * DeviceManager::findDeviceThread(void * pthis) {
+	DeviceManager *mgm = (DeviceManager*)pthis;
+	if (mgm!=NULL) {
+		mgm->findDevices();
+	}
+	return NULL;
+}
+
+void DeviceManager::findDevices() {
     // Remove active devices
     while (!gpsDeviceList.empty())
     {
@@ -101,20 +119,19 @@ void DeviceManager::startFindDevices() {
         delete(dev);
     }
 
-    FILE *mounts = NULL;
-    struct mntent *ent = NULL;
-    mounts = setmntent("/etc/mtab", "r");
-
+    bool scanMounted = true;
     bool searchGarmin = true;
     string backupPath = "";
     if (this->configuration != NULL) {
         TiXmlElement * pRoot = this->configuration->FirstChildElement( "GarminPlugin" );
         TiXmlElement * settings = NULL;
+        TiXmlElement * mounted = NULL;
         TiXmlElement * ftools = NULL;
         TiXmlElement * backup = NULL;
 
         if (pRoot != NULL) { settings = pRoot->FirstChildElement("Settings"); }
         if (settings != NULL) {
+        	mounted = settings->FirstChildElement("ScanMounted");
         	ftools = settings->FirstChildElement("ForerunnerTools");
         	backup = settings->FirstChildElement("BackupWorkouts");
         } else {
@@ -125,6 +142,11 @@ void DeviceManager::startFindDevices() {
         } else {
 			Log::dbg("Xml Element ForerunnerTools is null!");
 		}
+        if (mounted != NULL) {
+            scanMounted = getXmlBoolAttribute(mounted, "enabled", true);
+        } else {
+			Log::dbg("Xml Element ScanMounted is null!");
+		}
         if (backup != NULL) {
             bool doBackup = getXmlBoolAttribute(backup, "enabled", false);
             if (doBackup) {
@@ -137,20 +159,28 @@ void DeviceManager::startFindDevices() {
 		}
     }
 
-    Log::dbg("Searching for Edge705/Oregon300/...");
-    while ( (ent = getmntent(mounts)) != NULL ) {
-        string filesystype = ent->mnt_type;
-        string mountPath = ent->mnt_dir;
-        if (filesystype.compare("vfat") == 0) {
-            Log::dbg("Searching on ["+mountPath+"] ["+filesystype+"]");
-            GpsDevice *dev = createGarminDeviceFromPath(mountPath, NULL);
-            if (dev != NULL) {
-            	dev->setBackupPath(backupPath);
-                gpsDeviceList.push_back(dev);
+    if (scanMounted) {
+        FILE *mounts = NULL;
+        struct mntent *ent = NULL;
+        mounts = setmntent("/etc/mtab", "r");
+
+        Log::dbg("Searching for Edge705/Oregon300/...");
+        while ( (ent = getmntent(mounts)) != NULL ) {
+            string filesystype = ent->mnt_type;
+            string mountPath = ent->mnt_dir;
+            if (filesystype.compare("vfat") == 0) {
+                Log::dbg("Searching on ["+mountPath+"] ["+filesystype+"]");
+                GpsDevice *dev = createGarminDeviceFromPath(mountPath, NULL);
+                if (dev != NULL) {
+                    dev->setBackupPath(backupPath);
+                    gpsDeviceList.push_back(dev);
+                }
+            } else {
+                Log::dbg("Not searching on ["+mountPath+"] ["+filesystype+"] - wrong fstype.");
             }
-        } else {
-            Log::dbg("Not searching on ["+mountPath+"] ["+filesystype+"] - wrong fstype.");
         }
+    } else {
+        Log::dbg("Scanning for mounted devices is disabled!");
     }
 
     string deviceName;
@@ -267,6 +297,8 @@ void DeviceManager::startFindDevices() {
     std::ostringstream infoOut;
     infoOut << "Number of devices found: " << gpsDeviceList.size();
     Log::info(infoOut.str());
+
+    this->findDeviceState = 3;
 }
 
 void DeviceManager::setConfiguration(TiXmlDocument * config) {
@@ -276,11 +308,16 @@ void DeviceManager::setConfiguration(TiXmlDocument * config) {
 
 
 void DeviceManager::cancelFindDevices() {
-
+    Log::dbg("Cancel findDevice thread in DeviceManager");
+    if (this->threadId > 0) {
+        pthread_cancel(this->threadId);
+        this->threadId = 0;
+    }
+    this->findDeviceState = 0;
 }
 
 int DeviceManager::finishedFindDevices() {
-    return 1;
+    return this->findDeviceState;
 }
 
 GpsDevice * DeviceManager::getGpsDevice(int number)
