@@ -54,7 +54,7 @@ const char * pluginName = "Garmin Communicator";
 /**
  * A variable that stores the plugin description (may contain HTML)
  */
-const char * pluginDescription = "<a href=\"http://www.andreas-diesner.de/garminplugin/\">Garmin Communicator - Fake</a> plugin. Version 0.3.17";
+const char * pluginDescription = "<a href=\"http://www.andreas-diesner.de/garminplugin/\">Garmin Communicator - Fake</a> plugin. Version 0.3.20";
 
 /**
  * A variable that stores the mime description of the plugin.
@@ -484,10 +484,23 @@ bool methodCancelFindDevices(NPObject *obj, const NPVariant args[], uint32_t arg
  */
 bool methodFinishFindDevices(NPObject *obj, const NPVariant args[], uint32_t argCount, NPVariant * result)
 {
+    /*
+     * Unlike most other routines, this function knows only "0" or "1" as return value
+     * (not "0"=idle, "1"=working, "2"=waiting, "3"=finished)
+     *
+     * The deviceManager uses internally the state (0=idle, 1=working and 3=finished)
+     * which results in this confusing code: if state!=1 return 1     */
+
     result->type = NPVariantType_Int32;
-    result->value.intValue = devManager->finishedFindDevices();
-    printFinishState("FinishFindDevices", result->value.intValue);
-    return true;
+    int searchState = devManager->finishedFindDevices();
+    printFinishState("FinishFindDevices", searchState);
+    if (searchState != 1 /* = WORKING */ ) { 
+    	result->value.intValue = 1;   /* FINISHED SEARCHING */
+    } else {
+    	result->value.intValue = 0;   /* STILL SEARCHING */
+    }
+
+  	return true;
 }
 
 /**
@@ -1982,6 +1995,8 @@ static NPError nevv(NPMIMEType pluginType, NPP instance, uint16_t mode, int16_t
 
     npnfuncs->releaseobject(windowObject);
 
+    setlocale(LC_ALL, "POSIX");
+
 	return NPERR_NO_ERROR;
 }
 
